// Copyright (c) 2001-2004 Rowley Associates Limited.
//
// This file may be distributed under the terms of the License Agreement
// provided with this software.
//
// THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
////////////////////////////////////////////////////////////////////////////////
//
//                Philips LPC210X Timer Interrupt Example
//
// Description
// -----------
// This example demonstrates a simple interrupt handler and setting up a timer.
//
////////////////////////////////////////////////////////////////////////////////

#include <targets/LPC210x.h>

static int timer0Count, timer1Count;

/*
static unsigned int
processorClockFrequency(void)
{
  return OSCILLATOR_CLOCK_FREQUENCY * (PLLCON & 1 ? (PLLCFG & 0xF) + 1 : 1);
}

static unsigned int
peripheralClockFrequency(void)
{
  unsigned int divider;
  switch (VPBDIV & 3)
    {
      case 0:
        divider = 4;
        break;
      case 1:
        divider = 1;
        break;
      case 2:
        divider = 2;
        break;
    }
  return processorClockFrequency() / divider;
}*/

static void
delay(int n)
{
  volatile int i;
  for (i = 0; i < n; ++i);
}

static void
UARTInitialize(unsigned int baud)
{
  //unsigned int divisor = peripheralClockFrequency() / (16 * baud);
  unsigned int divisor = (14745600 / 4) / (16 * baud);
  U0LCR = 0x83; /* 8 bit, 1 stop bit, no parity, enable DLAB */
  U0DLL = divisor & 0xFF;
  U0DLM = (divisor >> 8) & 0xFF;
  U0LCR &= ~0x80; /* Disable DLAB */
  PINSEL0 = PINSEL0 & ~0xF | 0x5;
  U0FCR = 1;

  U0IER |= 1;

}

static void
UARTWriteChar(unsigned char ch)
{
  while ((U0LSR & 0x20) == 0);
  U0THR = ch;
}

static void
UARTWriteString(unsigned char *pch)
{
  while (*pch != 0) { UARTWriteChar (*pch++); } 
}

static void
UARTWriteHexUC(unsigned char ucValue)
{
  unsigned char ucNib;
  ucNib = ucValue >> 4;
  ucNib = ucNib > 9 ? ucNib - 10 + 0x41 : ucNib + 0x30;
  UARTWriteChar (ucNib);
  ucNib = ucValue & 0xf;
  ucNib = ucNib > 9 ? ucNib - 10 + 0x41 : ucNib + 0x30;
  UARTWriteChar (ucNib);
}

static void
UARTWriteHexUI(unsigned int uiValue)
{
  UARTWriteHexUC ((uiValue >>  8) & 0xff);
  UARTWriteHexUC ((uiValue >>  0) & 0xff);
}

static void
UARTWriteHexUL(unsigned long ulValue)
{
  UARTWriteHexUI ((ulValue >> 16) & 0xffff);
  UARTWriteHexUI ((ulValue >>  0) & 0xffff);
}

long charcount = 0;

static void uart0ISR(void) __attribute__ ((interrupt ("IRQ")));

static void uart0ISR(void)
{
  char cDummy, cIIR, cLSR;

  /* Clear the uart 0 interrupt */
  cDummy  = U0RBR;
  cIIR    = U0IIR;
  cLSR    = U0LSR;
  /* Update VIC priorities */
  charcount++;

  VICVectAddr = 0;
}


static void
ledInit()
{
  IODIR |= 0x00000080;
  IOSET = 0x00000080;
}

static void
ledOn(int led)
{
  IOCLR = led;
}

static void
ledOff(int led)
{
  IOSET = led;
}

static void timer0ISR(void) __attribute__ ((interrupt ("IRQ")));

static void
timer0ISR(void)
{
  if (++timer0Count % 2)
  IOCLR = 0x00000080;
  else
  IOSET = 0x00000080;
  /* Clear the timer 0 interrupt */
  T0IR = 0xFF;
  /* Update VIC priorities */
  VICVectAddr = 0;
}

static void timer1ISR(void) __attribute__ ((interrupt ("IRQ")));

static
void
timer1ISR(void)
{
  ++timer1Count;
  /* Clear the timer 1 interrupt */
  T1IR = 0xFF;
  /* Update VIC priorities */
  VICVectAddr = 0;
}

int
main(void)
{
  ledInit();

  /* Timer 0 interrupt is an IRQ interrupt */
  VICIntSelect &= ~0x10;
  /* Enable timer 0 interrupt */
  VICIntEnable = 0x10;
  /* Use slot 0 for timer 0 interrupt */
  VICVectCntl0 = 0x24;
  /* Set the address of ISR for slot 0 */
  VICVectAddr0 = (unsigned int)timer0ISR;

  /* Timer 1 interrupt is an IRQ interrupt */
  VICIntSelect &= ~0x20;
  /* Enable timer 1 interrupt */
  VICIntEnable = 0x20;
  /* Use slot 1 for timer 1 interrupt */
  VICVectCntl1 = 0x25;
  /* Set the address of ISR for slot 1 */
  VICVectAddr1 = (unsigned int)timer1ISR;

  /* Timer 1 interrupt is an IRQ interrupt */
  VICIntSelect &= ~0x40;
  /* Enable timer 1 interrupt */
  VICIntEnable = 0x40;
  /* Use slot 1 for timer 1 interrupt */
  VICVectCntl2 = 0x26;
  /* Set the address of ISR for slot 1 */
  VICVectAddr2 = (unsigned int)uart0ISR;


  /* Reset timer 0 */
  T0TCR = 0;
  /* Set the timer 0 prescale counter */
  T0PR = 2500;
  /* Set timer 0 match register */
  T0MR0 = 1000;
  /* Generate interrupt and reset counter on match */
  T0MCR = 3;
  /* Start timer 0 */
  T0TCR = 1;

  /* Reset timer 1 */
  T1TCR = 0;
  /* Set the timer 1 prescale counter */
  T1PR = 2500;
  /* Set timer 1 match register */
  T1MR0 = 1000;
  /* Generate interrupt and reset counter on match */
  T1MCR = 3;
  /* Start timer 1 */
  T1TCR = 1;

  UARTInitialize (9600);

  UARTWriteString ("Hallo Welt");

  /* Enable Interrupts */
  __ARMLIB_enableIRQ();

  //while (timer0Count < 5 && timer1Count < 5);

  while (1)
  {
    UARTWriteString ("Hallo Welt ");
    UARTWriteHexUL (timer0Count);
    UARTWriteString ("  ");
    UARTWriteHexUL (charcount);
    UARTWriteString ("\n\r");
  }

  return 0; 
}
